"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class Authorization(TLObject):
    """TL type: auth.authorization"""
    CONSTRUCTOR_ID = 0x2ea2c0d4
    SUBCLASS_OF_ID = 0xb9e04e39

    def __init__(self, user: 'TypeUser', setup_password_required: Optional[bool] = None, otherwise_relogin_days: Optional[int] = None, tmp_sessions: Optional[int] = None, future_auth_token: Optional[bytes] = None):
        self.user = user
        self.setup_password_required = setup_password_required
        self.otherwise_relogin_days = otherwise_relogin_days
        self.tmp_sessions = tmp_sessions
        self.future_auth_token = future_auth_token

    def to_dict(self):
        return {
            '_': 'Authorization',
            'user': self.user,
            'setup_password_required': self.setup_password_required,
            'otherwise_relogin_days': self.otherwise_relogin_days,
            'tmp_sessions': self.tmp_sessions,
            'future_auth_token': self.future_auth_token,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.otherwise_relogin_days is not None:
            flags |= (1 << 1)
        if self.tmp_sessions is not None:
            flags |= (1 << 0)
        if self.future_auth_token is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.user))
        if self.otherwise_relogin_days is not None:
            buf.write(struct.pack('<i', self.otherwise_relogin_days))
        if self.tmp_sessions is not None:
            buf.write(struct.pack('<i', self.tmp_sessions))
        if self.future_auth_token is not None:
            buf.write(TLObject.serialize_bytes(self.future_auth_token))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['setup_password_required'] = bool(flags & (1 << 1))
        if flags & (1 << 1):
            _val_otherwise_relogin_days = reader.read_int()
            _args['otherwise_relogin_days'] = _val_otherwise_relogin_days
        else:
            _args['otherwise_relogin_days'] = None
        if flags & (1 << 0):
            _val_tmp_sessions = reader.read_int()
            _args['tmp_sessions'] = _val_tmp_sessions
        else:
            _args['tmp_sessions'] = None
        if flags & (1 << 2):
            _val_future_auth_token = reader.tgread_bytes()
            _args['future_auth_token'] = _val_future_auth_token
        else:
            _args['future_auth_token'] = None
        _val_user = reader.tgread_object()
        _args['user'] = _val_user
        return cls(**_args)


class AuthorizationSignUpRequired(TLObject):
    """TL type: auth.authorizationSignUpRequired"""
    CONSTRUCTOR_ID = 0x44747e9a
    SUBCLASS_OF_ID = 0xb9e04e39

    def __init__(self, terms_of_service: Optional['TypeTermsOfService'] = None):
        self.terms_of_service = terms_of_service

    def to_dict(self):
        return {
            '_': 'AuthorizationSignUpRequired',
            'terms_of_service': self.terms_of_service,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.terms_of_service is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        if self.terms_of_service is not None:
            buf.write(bytes(self.terms_of_service))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_terms_of_service = reader.tgread_object()
            _args['terms_of_service'] = _val_terms_of_service
        else:
            _args['terms_of_service'] = None
        return cls(**_args)


class CodeTypeCall(TLObject):
    """TL type: auth.codeTypeCall"""
    CONSTRUCTOR_ID = 0x741cd3e3
    SUBCLASS_OF_ID = 0xb3f3e401

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'CodeTypeCall',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class CodeTypeFlashCall(TLObject):
    """TL type: auth.codeTypeFlashCall"""
    CONSTRUCTOR_ID = 0x226ccefb
    SUBCLASS_OF_ID = 0xb3f3e401

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'CodeTypeFlashCall',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class CodeTypeFragmentSms(TLObject):
    """TL type: auth.codeTypeFragmentSms"""
    CONSTRUCTOR_ID = 0x06ed998c
    SUBCLASS_OF_ID = 0xb3f3e401

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'CodeTypeFragmentSms',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class CodeTypeMissedCall(TLObject):
    """TL type: auth.codeTypeMissedCall"""
    CONSTRUCTOR_ID = 0xd61ad6ee
    SUBCLASS_OF_ID = 0xb3f3e401

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'CodeTypeMissedCall',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class CodeTypeSms(TLObject):
    """TL type: auth.codeTypeSms"""
    CONSTRUCTOR_ID = 0x72a3158c
    SUBCLASS_OF_ID = 0xb3f3e401

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'CodeTypeSms',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class ExportedAuthorization(TLObject):
    """TL type: auth.exportedAuthorization"""
    CONSTRUCTOR_ID = 0xb434e2b8
    SUBCLASS_OF_ID = 0x5fd1ec51

    def __init__(self, id: int, bytes: bytes):
        self.id = id
        self.bytes = bytes

    def to_dict(self):
        return {
            '_': 'ExportedAuthorization',
            'id': self.id,
            'bytes': self.bytes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.id))
        buf.write(TLObject.serialize_bytes(self.bytes))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_id = reader.read_long()
        _args['id'] = _val_id
        _val_bytes = reader.tgread_bytes()
        _args['bytes'] = _val_bytes
        return cls(**_args)


class LoggedOut(TLObject):
    """TL type: auth.loggedOut"""
    CONSTRUCTOR_ID = 0xc3a2835f
    SUBCLASS_OF_ID = 0x0a804315

    def __init__(self, future_auth_token: Optional[bytes] = None):
        self.future_auth_token = future_auth_token

    def to_dict(self):
        return {
            '_': 'LoggedOut',
            'future_auth_token': self.future_auth_token,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.future_auth_token is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        if self.future_auth_token is not None:
            buf.write(TLObject.serialize_bytes(self.future_auth_token))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_future_auth_token = reader.tgread_bytes()
            _args['future_auth_token'] = _val_future_auth_token
        else:
            _args['future_auth_token'] = None
        return cls(**_args)


class LoginToken(TLObject):
    """TL type: auth.loginToken"""
    CONSTRUCTOR_ID = 0x629f1980
    SUBCLASS_OF_ID = 0x6b55f636

    def __init__(self, expires: Optional[datetime], token: bytes):
        self.expires = expires
        self.token = token

    def to_dict(self):
        return {
            '_': 'LoginToken',
            'expires': self.expires,
            'token': self.token,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.expires))
        buf.write(TLObject.serialize_bytes(self.token))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_expires = reader.read_int()
        _args['expires'] = _val_expires
        _val_token = reader.tgread_bytes()
        _args['token'] = _val_token
        return cls(**_args)


class LoginTokenMigrateTo(TLObject):
    """TL type: auth.loginTokenMigrateTo"""
    CONSTRUCTOR_ID = 0x068e9916
    SUBCLASS_OF_ID = 0x6b55f636

    def __init__(self, dc_id: int, token: bytes):
        self.dc_id = dc_id
        self.token = token

    def to_dict(self):
        return {
            '_': 'LoginTokenMigrateTo',
            'dc_id': self.dc_id,
            'token': self.token,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.dc_id))
        buf.write(TLObject.serialize_bytes(self.token))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_dc_id = reader.read_int()
        _args['dc_id'] = _val_dc_id
        _val_token = reader.tgread_bytes()
        _args['token'] = _val_token
        return cls(**_args)


class LoginTokenSuccess(TLObject):
    """TL type: auth.loginTokenSuccess"""
    CONSTRUCTOR_ID = 0x390d5c5e
    SUBCLASS_OF_ID = 0x6b55f636

    def __init__(self, authorization: 'TypeAuthorization'):
        self.authorization = authorization

    def to_dict(self):
        return {
            '_': 'LoginTokenSuccess',
            'authorization': self.authorization,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.authorization))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_authorization = reader.tgread_object()
        _args['authorization'] = _val_authorization
        return cls(**_args)


class PasskeyLoginOptions(TLObject):
    """TL type: auth.passkeyLoginOptions"""
    CONSTRUCTOR_ID = 0xe2037789
    SUBCLASS_OF_ID = 0xd9793032

    def __init__(self, options: 'TypeDataJSON'):
        self.options = options

    def to_dict(self):
        return {
            '_': 'PasskeyLoginOptions',
            'options': self.options,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.options))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_options = reader.tgread_object()
        _args['options'] = _val_options
        return cls(**_args)


class PasswordRecovery(TLObject):
    """TL type: auth.passwordRecovery"""
    CONSTRUCTOR_ID = 0x137948a5
    SUBCLASS_OF_ID = 0xfa72d43a

    def __init__(self, email_pattern: str):
        self.email_pattern = email_pattern

    def to_dict(self):
        return {
            '_': 'PasswordRecovery',
            'email_pattern': self.email_pattern,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.email_pattern))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_email_pattern = reader.tgread_string()
        _args['email_pattern'] = _val_email_pattern
        return cls(**_args)


class SentCode(TLObject):
    """TL type: auth.sentCode"""
    CONSTRUCTOR_ID = 0x5e002502
    SUBCLASS_OF_ID = 0x6ce87081

    def __init__(self, type: 'TypeSentCodeType', phone_code_hash: str, next_type: Optional['TypeCodeType'] = None, timeout: Optional[int] = None):
        self.type = type
        self.phone_code_hash = phone_code_hash
        self.next_type = next_type
        self.timeout = timeout

    def to_dict(self):
        return {
            '_': 'SentCode',
            'type': self.type,
            'phone_code_hash': self.phone_code_hash,
            'next_type': self.next_type,
            'timeout': self.timeout,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_type is not None:
            flags |= (1 << 1)
        if self.timeout is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.type))
        buf.write(TLObject.serialize_bytes(self.phone_code_hash))
        if self.next_type is not None:
            buf.write(bytes(self.next_type))
        if self.timeout is not None:
            buf.write(struct.pack('<i', self.timeout))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_type = reader.tgread_object()
        _args['type'] = _val_type
        _val_phone_code_hash = reader.tgread_string()
        _args['phone_code_hash'] = _val_phone_code_hash
        if flags & (1 << 1):
            _val_next_type = reader.tgread_object()
            _args['next_type'] = _val_next_type
        else:
            _args['next_type'] = None
        if flags & (1 << 2):
            _val_timeout = reader.read_int()
            _args['timeout'] = _val_timeout
        else:
            _args['timeout'] = None
        return cls(**_args)


class SentCodePaymentRequired(TLObject):
    """TL type: auth.sentCodePaymentRequired"""
    CONSTRUCTOR_ID = 0xe0955a3c
    SUBCLASS_OF_ID = 0x6ce87081

    def __init__(self, store_product: str, phone_code_hash: str, support_email_address: str, support_email_subject: str, currency: str, amount: int):
        self.store_product = store_product
        self.phone_code_hash = phone_code_hash
        self.support_email_address = support_email_address
        self.support_email_subject = support_email_subject
        self.currency = currency
        self.amount = amount

    def to_dict(self):
        return {
            '_': 'SentCodePaymentRequired',
            'store_product': self.store_product,
            'phone_code_hash': self.phone_code_hash,
            'support_email_address': self.support_email_address,
            'support_email_subject': self.support_email_subject,
            'currency': self.currency,
            'amount': self.amount,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.store_product))
        buf.write(TLObject.serialize_bytes(self.phone_code_hash))
        buf.write(TLObject.serialize_bytes(self.support_email_address))
        buf.write(TLObject.serialize_bytes(self.support_email_subject))
        buf.write(TLObject.serialize_bytes(self.currency))
        buf.write(struct.pack('<q', self.amount))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_store_product = reader.tgread_string()
        _args['store_product'] = _val_store_product
        _val_phone_code_hash = reader.tgread_string()
        _args['phone_code_hash'] = _val_phone_code_hash
        _val_support_email_address = reader.tgread_string()
        _args['support_email_address'] = _val_support_email_address
        _val_support_email_subject = reader.tgread_string()
        _args['support_email_subject'] = _val_support_email_subject
        _val_currency = reader.tgread_string()
        _args['currency'] = _val_currency
        _val_amount = reader.read_long()
        _args['amount'] = _val_amount
        return cls(**_args)


class SentCodeSuccess(TLObject):
    """TL type: auth.sentCodeSuccess"""
    CONSTRUCTOR_ID = 0x2390fe44
    SUBCLASS_OF_ID = 0x6ce87081

    def __init__(self, authorization: 'TypeAuthorization'):
        self.authorization = authorization

    def to_dict(self):
        return {
            '_': 'SentCodeSuccess',
            'authorization': self.authorization,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.authorization))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_authorization = reader.tgread_object()
        _args['authorization'] = _val_authorization
        return cls(**_args)


class SentCodeTypeApp(TLObject):
    """TL type: auth.sentCodeTypeApp"""
    CONSTRUCTOR_ID = 0x3dbb5986
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, length: int):
        self.length = length

    def to_dict(self):
        return {
            '_': 'SentCodeTypeApp',
            'length': self.length,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.length))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_length = reader.read_int()
        _args['length'] = _val_length
        return cls(**_args)


class SentCodeTypeCall(TLObject):
    """TL type: auth.sentCodeTypeCall"""
    CONSTRUCTOR_ID = 0x5353e5a7
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, length: int):
        self.length = length

    def to_dict(self):
        return {
            '_': 'SentCodeTypeCall',
            'length': self.length,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.length))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_length = reader.read_int()
        _args['length'] = _val_length
        return cls(**_args)


class SentCodeTypeEmailCode(TLObject):
    """TL type: auth.sentCodeTypeEmailCode"""
    CONSTRUCTOR_ID = 0xf450f59b
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, email_pattern: str, length: int, apple_signin_allowed: Optional[bool] = None, google_signin_allowed: Optional[bool] = None, reset_available_period: Optional[int] = None, reset_pending_date: Optional[datetime] = None):
        self.email_pattern = email_pattern
        self.length = length
        self.apple_signin_allowed = apple_signin_allowed
        self.google_signin_allowed = google_signin_allowed
        self.reset_available_period = reset_available_period
        self.reset_pending_date = reset_pending_date

    def to_dict(self):
        return {
            '_': 'SentCodeTypeEmailCode',
            'email_pattern': self.email_pattern,
            'length': self.length,
            'apple_signin_allowed': self.apple_signin_allowed,
            'google_signin_allowed': self.google_signin_allowed,
            'reset_available_period': self.reset_available_period,
            'reset_pending_date': self.reset_pending_date,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.apple_signin_allowed:
            flags |= (1 << 0)
        if self.google_signin_allowed:
            flags |= (1 << 1)
        if self.reset_available_period is not None:
            flags |= (1 << 3)
        if self.reset_pending_date is not None:
            flags |= (1 << 4)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.email_pattern))
        buf.write(struct.pack('<i', self.length))
        if self.reset_available_period is not None:
            buf.write(struct.pack('<i', self.reset_available_period))
        if self.reset_pending_date is not None:
            buf.write(struct.pack('<i', self.reset_pending_date))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['apple_signin_allowed'] = bool(flags & (1 << 0))
        _args['google_signin_allowed'] = bool(flags & (1 << 1))
        _val_email_pattern = reader.tgread_string()
        _args['email_pattern'] = _val_email_pattern
        _val_length = reader.read_int()
        _args['length'] = _val_length
        if flags & (1 << 3):
            _val_reset_available_period = reader.read_int()
            _args['reset_available_period'] = _val_reset_available_period
        else:
            _args['reset_available_period'] = None
        if flags & (1 << 4):
            _val_reset_pending_date = reader.read_int()
            _args['reset_pending_date'] = _val_reset_pending_date
        else:
            _args['reset_pending_date'] = None
        return cls(**_args)


class SentCodeTypeFirebaseSms(TLObject):
    """TL type: auth.sentCodeTypeFirebaseSms"""
    CONSTRUCTOR_ID = 0x009fd736
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, length: int, nonce: Optional[bytes] = None, play_integrity_project_id: Optional[int] = None, play_integrity_nonce: Optional[bytes] = None, receipt: Optional[str] = None, push_timeout: Optional[int] = None):
        self.length = length
        self.nonce = nonce
        self.play_integrity_project_id = play_integrity_project_id
        self.play_integrity_nonce = play_integrity_nonce
        self.receipt = receipt
        self.push_timeout = push_timeout

    def to_dict(self):
        return {
            '_': 'SentCodeTypeFirebaseSms',
            'length': self.length,
            'nonce': self.nonce,
            'play_integrity_project_id': self.play_integrity_project_id,
            'play_integrity_nonce': self.play_integrity_nonce,
            'receipt': self.receipt,
            'push_timeout': self.push_timeout,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.nonce is not None:
            flags |= (1 << 0)
        if self.play_integrity_nonce is not None:
            flags |= (1 << 2)
        if self.push_timeout is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.length))
        if self.nonce is not None:
            buf.write(TLObject.serialize_bytes(self.nonce))
        if self.play_integrity_project_id is not None:
            buf.write(struct.pack('<q', self.play_integrity_project_id))
        if self.play_integrity_nonce is not None:
            buf.write(TLObject.serialize_bytes(self.play_integrity_nonce))
        if self.receipt is not None:
            buf.write(TLObject.serialize_bytes(self.receipt))
        if self.push_timeout is not None:
            buf.write(struct.pack('<i', self.push_timeout))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_nonce = reader.tgread_bytes()
            _args['nonce'] = _val_nonce
        else:
            _args['nonce'] = None
        if flags & (1 << 2):
            _val_play_integrity_project_id = reader.read_long()
            _args['play_integrity_project_id'] = _val_play_integrity_project_id
        else:
            _args['play_integrity_project_id'] = None
        if flags & (1 << 2):
            _val_play_integrity_nonce = reader.tgread_bytes()
            _args['play_integrity_nonce'] = _val_play_integrity_nonce
        else:
            _args['play_integrity_nonce'] = None
        if flags & (1 << 1):
            _val_receipt = reader.tgread_string()
            _args['receipt'] = _val_receipt
        else:
            _args['receipt'] = None
        if flags & (1 << 1):
            _val_push_timeout = reader.read_int()
            _args['push_timeout'] = _val_push_timeout
        else:
            _args['push_timeout'] = None
        _val_length = reader.read_int()
        _args['length'] = _val_length
        return cls(**_args)


class SentCodeTypeFlashCall(TLObject):
    """TL type: auth.sentCodeTypeFlashCall"""
    CONSTRUCTOR_ID = 0xab03c6d9
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, pattern: str):
        self.pattern = pattern

    def to_dict(self):
        return {
            '_': 'SentCodeTypeFlashCall',
            'pattern': self.pattern,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.pattern))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_pattern = reader.tgread_string()
        _args['pattern'] = _val_pattern
        return cls(**_args)


class SentCodeTypeFragmentSms(TLObject):
    """TL type: auth.sentCodeTypeFragmentSms"""
    CONSTRUCTOR_ID = 0xd9565c39
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, url: str, length: int):
        self.url = url
        self.length = length

    def to_dict(self):
        return {
            '_': 'SentCodeTypeFragmentSms',
            'url': self.url,
            'length': self.length,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.url))
        buf.write(struct.pack('<i', self.length))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_url = reader.tgread_string()
        _args['url'] = _val_url
        _val_length = reader.read_int()
        _args['length'] = _val_length
        return cls(**_args)


class SentCodeTypeMissedCall(TLObject):
    """TL type: auth.sentCodeTypeMissedCall"""
    CONSTRUCTOR_ID = 0x82006484
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, prefix: str, length: int):
        self.prefix = prefix
        self.length = length

    def to_dict(self):
        return {
            '_': 'SentCodeTypeMissedCall',
            'prefix': self.prefix,
            'length': self.length,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.prefix))
        buf.write(struct.pack('<i', self.length))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_prefix = reader.tgread_string()
        _args['prefix'] = _val_prefix
        _val_length = reader.read_int()
        _args['length'] = _val_length
        return cls(**_args)


class SentCodeTypeSetUpEmailRequired(TLObject):
    """TL type: auth.sentCodeTypeSetUpEmailRequired"""
    CONSTRUCTOR_ID = 0xa5491dea
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, apple_signin_allowed: Optional[bool] = None, google_signin_allowed: Optional[bool] = None):
        self.apple_signin_allowed = apple_signin_allowed
        self.google_signin_allowed = google_signin_allowed

    def to_dict(self):
        return {
            '_': 'SentCodeTypeSetUpEmailRequired',
            'apple_signin_allowed': self.apple_signin_allowed,
            'google_signin_allowed': self.google_signin_allowed,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.apple_signin_allowed:
            flags |= (1 << 0)
        if self.google_signin_allowed:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['apple_signin_allowed'] = bool(flags & (1 << 0))
        _args['google_signin_allowed'] = bool(flags & (1 << 1))
        return cls(**_args)


class SentCodeTypeSms(TLObject):
    """TL type: auth.sentCodeTypeSms"""
    CONSTRUCTOR_ID = 0xc000bba2
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, length: int):
        self.length = length

    def to_dict(self):
        return {
            '_': 'SentCodeTypeSms',
            'length': self.length,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.length))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_length = reader.read_int()
        _args['length'] = _val_length
        return cls(**_args)


class SentCodeTypeSmsPhrase(TLObject):
    """TL type: auth.sentCodeTypeSmsPhrase"""
    CONSTRUCTOR_ID = 0xb37794af
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, beginning: Optional[str] = None):
        self.beginning = beginning

    def to_dict(self):
        return {
            '_': 'SentCodeTypeSmsPhrase',
            'beginning': self.beginning,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.beginning is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        if self.beginning is not None:
            buf.write(TLObject.serialize_bytes(self.beginning))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_beginning = reader.tgread_string()
            _args['beginning'] = _val_beginning
        else:
            _args['beginning'] = None
        return cls(**_args)


class SentCodeTypeSmsWord(TLObject):
    """TL type: auth.sentCodeTypeSmsWord"""
    CONSTRUCTOR_ID = 0xa416ac81
    SUBCLASS_OF_ID = 0xff5b158e

    def __init__(self, beginning: Optional[str] = None):
        self.beginning = beginning

    def to_dict(self):
        return {
            '_': 'SentCodeTypeSmsWord',
            'beginning': self.beginning,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.beginning is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        if self.beginning is not None:
            buf.write(TLObject.serialize_bytes(self.beginning))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_beginning = reader.tgread_string()
            _args['beginning'] = _val_beginning
        else:
            _args['beginning'] = None
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class GetChannelDifferenceRequest(TLRequest):
    """TL type: updates.getChannelDifference"""
    CONSTRUCTOR_ID = 0x03173d78
    SUBCLASS_OF_ID = 0x29896f5d

    def __init__(self, channel: 'TypeInputChannel', filter: 'TypeChannelMessagesFilter', pts: int, limit: int, force: Optional[bool] = None):
        self.channel = channel
        self.filter = filter
        self.pts = pts
        self.limit = limit
        self.force = force

    def to_dict(self):
        return {
            '_': 'GetChannelDifferenceRequest',
            'channel': self.channel,
            'filter': self.filter,
            'pts': self.pts,
            'limit': self.limit,
            'force': self.force,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.force:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        buf.write(bytes(self.filter))
        buf.write(struct.pack('<i', self.pts))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['force'] = bool(flags & (1 << 0))
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_filter = reader.tgread_object()
        _args['filter'] = _val_filter
        _val_pts = reader.read_int()
        _args['pts'] = _val_pts
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class GetDifferenceRequest(TLRequest):
    """TL type: updates.getDifference"""
    CONSTRUCTOR_ID = 0x19c2f763
    SUBCLASS_OF_ID = 0x20482874

    def __init__(self, pts: int, date: Optional[datetime], qts: int, pts_limit: Optional[int] = None, pts_total_limit: Optional[int] = None, qts_limit: Optional[int] = None):
        self.pts = pts
        self.date = date
        self.qts = qts
        self.pts_limit = pts_limit
        self.pts_total_limit = pts_total_limit
        self.qts_limit = qts_limit

    def to_dict(self):
        return {
            '_': 'GetDifferenceRequest',
            'pts': self.pts,
            'date': self.date,
            'qts': self.qts,
            'pts_limit': self.pts_limit,
            'pts_total_limit': self.pts_total_limit,
            'qts_limit': self.qts_limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.pts_limit is not None:
            flags |= (1 << 1)
        if self.pts_total_limit is not None:
            flags |= (1 << 0)
        if self.qts_limit is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.pts))
        buf.write(struct.pack('<i', self.date))
        buf.write(struct.pack('<i', self.qts))
        if self.pts_limit is not None:
            buf.write(struct.pack('<i', self.pts_limit))
        if self.pts_total_limit is not None:
            buf.write(struct.pack('<i', self.pts_total_limit))
        if self.qts_limit is not None:
            buf.write(struct.pack('<i', self.qts_limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_pts = reader.read_int()
        _args['pts'] = _val_pts
        if flags & (1 << 1):
            _val_pts_limit = reader.read_int()
            _args['pts_limit'] = _val_pts_limit
        else:
            _args['pts_limit'] = None
        if flags & (1 << 0):
            _val_pts_total_limit = reader.read_int()
            _args['pts_total_limit'] = _val_pts_total_limit
        else:
            _args['pts_total_limit'] = None
        _val_date = reader.read_int()
        _args['date'] = _val_date
        _val_qts = reader.read_int()
        _args['qts'] = _val_qts
        if flags & (1 << 2):
            _val_qts_limit = reader.read_int()
            _args['qts_limit'] = _val_qts_limit
        else:
            _args['qts_limit'] = None
        return cls(**_args)


class GetStateRequest(TLRequest):
    """TL type: updates.getState"""
    CONSTRUCTOR_ID = 0xedd4882a
    SUBCLASS_OF_ID = 0x23df1a01

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetStateRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class DeleteFolderRequest(TLRequest):
    """TL type: folders.deleteFolder"""
    CONSTRUCTOR_ID = 0x1c295881
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, folder_id: int):
        self.folder_id = folder_id

    def to_dict(self):
        return {
            '_': 'DeleteFolderRequest',
            'folder_id': self.folder_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.folder_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_folder_id = reader.read_int()
        _args['folder_id'] = _val_folder_id
        return cls(**_args)


class EditPeerFoldersRequest(TLRequest):
    """TL type: folders.editPeerFolders"""
    CONSTRUCTOR_ID = 0x6847d0ab
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, folder_peers: List['TypeInputFolderPeer']):
        self.folder_peers = folder_peers

    def to_dict(self):
        return {
            '_': 'EditPeerFoldersRequest',
            'folder_peers': self.folder_peers,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.folder_peers)))
        for item in self.folder_peers:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_folder_peers = reader.read_int()
        _list_folder_peers = []
        for _ in range(_count_folder_peers):
            _item_folder_peers = reader.tgread_object()
            _list_folder_peers.append(_item_folder_peers)
        _args['folder_peers'] = _list_folder_peers
        return cls(**_args)




"""In-memory session implementation."""
from ..crypto import AuthKey


class MemorySession:
    """Stores session data in memory (lost on exit)."""

    def __init__(self):
        self._dc_id = 2
        self._server_address = '149.154.167.51'
        self._port = 443
        self._auth_key = AuthKey(None)
        self._takeout_id = None

    @property
    def dc_id(self): return self._dc_id

    @dc_id.setter
    def dc_id(self, value):
        self._dc_id = value

    @property
    def server_address(self): return self._server_address

    @property
    def port(self): return self._port

    @property
    def auth_key(self): return self._auth_key

    @auth_key.setter
    def auth_key(self, value):
        self._auth_key = value

    def set_dc(self, dc_id: int, server_address: str, port: int):
        self._dc_id = dc_id
        self._server_address = server_address
        self._port = port

    def save(self):
        pass  # Nothing to persist in memory

    def close(self):
        pass

    def clone(self, to_instance=None):
        cloned = to_instance or MemorySession()
        cloned._dc_id = self._dc_id
        cloned._server_address = self._server_address
        cloned._port = self._port
        cloned._auth_key = self._auth_key
        cloned._takeout_id = self._takeout_id
        return cloned

    def get_update_state(self, entity_id: int):
        return None

    def set_update_state(self, entity_id: int, state):
        pass

    def get_entity_rows_by_phone(self, phone: str):
        return None

    def get_entity_rows_by_username(self, username: str):
        return None

    def get_entity_rows_by_name(self, first_name: str, last_name: str):
        return None

    def get_entity_rows_by_id(self, entity_id: int, exact: bool = True):
        return None

    def process_entities(self, tlo):
        pass

    def cache_file(self, md5_digest, file_size, instance):
        pass

    def get_file(self, md5_digest, file_size, cls):
        return None



"""MessageContainer TL core type."""
import struct
from ..tlobject import TLObject


class MessageContainer(TLObject):
    CONSTRUCTOR_ID = 0x73f1f8dc
    MAXIMUM_SIZE = 1044456
    MAXIMUM_LENGTH = 100

    def __init__(self, messages):
        self.messages = messages

    @classmethod
    def from_reader(cls, reader):
        from .tlmessage import TLMessage
        count = reader.read_int()
        messages = []
        for _ in range(count):
            msg_id = reader.read_long()
            seq_no = reader.read_int()
            _length = reader.read_int()
            obj = reader.tgread_object()
            messages.append(TLMessage(msg_id, seq_no, obj))
        return cls(messages)

    def to_dict(self):
        return {'_': 'MessageContainer', 'messages': self.messages}



"""
MessagePacker - batches outgoing requests into MessageContainer when possible.
"""
import asyncio
import struct
from ..tl.core.messagecontainer import MessageContainer
from ..tl.core.gzippacked import GzipPacked


class MessagePacker:
    """
    Queues outgoing requests and packs multiple into a single
    MessageContainer when possible for efficiency.
    """

    def __init__(self, state, *, loggers):
        self._state = state
        import logging
        self._log = loggers.get(__name__) or logging.getLogger(__name__)
        self._deque = asyncio.Queue()

    def append(self, state):
        self._deque.put_nowait(state)

    def extend(self, states):
        for s in states:
            self.append(s)

    async def get(self):
        """
        Collects pending requests and packs them.
        Returns (data_bytes, [RequestState, ...]) or (None, []).
        """
        states = []
        data = bytearray()

        # Wait for at least one
        first = await self._deque.get()
        if first is None:
            return None, []

        states.append(first)

        # Drain any additional waiting without blocking
        while True:
            try:
                item = self._deque.get_nowait()
                if item is None:
                    break
                states.append(item)
            except asyncio.QueueEmpty:
                break

            if len(states) >= MessageContainer.MAXIMUM_LENGTH:
                break

        # Build the payload
        # Single message - no container needed
        if len(states) == 1:
            state = states[0]
            msg_id = self._state.write_data_as_message(
                data,
                bytes(state.request),
                isinstance(state.request, type) or state.request.CONSTRUCTOR_ID is not None,
                after_id=state.after.msg_id if state.after else None
            )
            state.msg_id = msg_id
            self._log.debug('Sending single %s', type(state.request).__name__)
            return bytes(data), states

        # Multiple messages - wrap in container
        container_buf = bytearray()
        for state in states:
            before = len(container_buf)
            msg_id = self._state.write_data_as_message(
                container_buf,
                bytes(state.request),
                True,
                after_id=state.after.msg_id if state.after else None
            )
            state.msg_id = msg_id

            if len(container_buf) > MessageContainer.MAXIMUM_SIZE:
                # Too large - remove last item and break
                container_buf = container_buf[:before]
                states = states[:-1]
                # Put the overflowed state back
                self._deque.put_nowait(states[-1])
                break

        # Write container header
        data.extend(struct.pack('<I', MessageContainer.CONSTRUCTOR_ID))
        data.extend(struct.pack('<i', len(states)))
        data.extend(container_buf)

        self._log.debug('Sending container with %d messages', len(states))
        return bytes(data), states

    def put_nowait(self, item):
        self._deque.put_nowait(item)



"""
Code generator: takes parsed TLObjects and generates Python source files
for tglib/tl/types/ and tglib/tl/functions/.
"""
import os
import struct
from collections import defaultdict
from pathlib import Path
from typing import List

from .parser import TLObject, TLArg

AUTO_GEN_NOTICE = '"""File auto-generated by tglib TL generator. Do not edit manually."""'

BASE_TYPES = {'string', 'bytes', 'int', 'long', 'int128', 'int256',
              'double', 'Bool', 'true', 'date'}


def _write_imports(f, depth: int, is_function: bool, tlobjects: List[TLObject]):
    """Write module-level imports."""
    base = '.' * depth
    f.write(f'{AUTO_GEN_NOTICE}\n')
    f.write(f'from {base}tl.tlobject import TLObject\n')
    if is_function:
        f.write(f'from {base}tl.tlobject import TLRequest\n')
    f.write('from typing import Optional, List, Union, TYPE_CHECKING\n')
    f.write('import os\n')
    f.write('import struct\n')
    f.write('from datetime import datetime\n')
    f.write('\n')


def _args_serialization(arg: TLArg, indent: str) -> List[str]:
    """Generate bytes serialization lines for one argument."""
    lines = []
    name = arg.name
    t = arg.type if not arg.flag_indicator else '#'

    def write(code):
        lines.append(f'{indent}{code}')

    if arg.flag_indicator:
        # Will be computed dynamically
        write(f'# flags computed below')
        return lines

    if arg.flag:
        # 'true' flag types are only reflected in the flags bitmask, never as fields
        if arg.type == 'true':
            return lines
        # Conditional field: only written if flag is set
        write(f'if self.{name} is not None:')
        sub = indent + '    '
        sub_lines = _serialize_single_type(name, arg.type, arg.is_vector,
                                           arg.use_vector_id,
                                           arg.skip_constructor_id, sub)
        if not sub_lines:
            lines.append(f'{sub}pass')
        else:
            lines.extend(sub_lines)
    else:
        sub_lines = _serialize_single_type(name, t, arg.is_vector,
                                           arg.use_vector_id,
                                           arg.skip_constructor_id, indent)
        lines.extend(sub_lines)

    return lines


def _serialize_single_type(name, t, is_vector, use_vector_id,
                            skip_constructor_id, indent) -> List[str]:
    lines = []

    def w(code):
        lines.append(f'{indent}{code}')

    if is_vector:
        if use_vector_id:
            w(f'buf.write(struct.pack(\'<i\', 0x1cb5c415))  # vector id')
        w(f'buf.write(struct.pack(\'<i\', len(self.{name})))')
        w(f'for item in self.{name}:')
        item_lines = _serialize_single_type('item', t, False, False,
                                            skip_constructor_id, indent + '    ')
        lines.extend(item_lines)
        return lines

    if t in ('int', 'date'):
        w(f'buf.write(struct.pack(\'<i\', self.{name}))')
    elif t == 'long':
        w(f'buf.write(struct.pack(\'<q\', self.{name}))')
    elif t == 'int128':
        w(f'buf.write(self.{name}.to_bytes(16, \'little\'))')
    elif t == 'int256':
        w(f'buf.write(self.{name}.to_bytes(32, \'little\'))')
    elif t == 'double':
        w(f'buf.write(struct.pack(\'<d\', self.{name}))')
    elif t in ('string', 'bytes'):
        w(f'buf.write(TLObject.serialize_bytes(self.{name}))')
    elif t == 'Bool':
        w(f'buf.write(struct.pack(\'<I\', 0x997275b5 if self.{name} else 0xbc799737))')
    elif t == 'true':
        pass  # true flags are only present/absent, no serialization needed
    else:
        if skip_constructor_id:
            w(f'buf.write(bytes(self.{name}))')
        else:
            w(f'buf.write(bytes(self.{name}))')

    return lines


def _write_to_bytes(obj: TLObject, f):
    """Write the _bytes() method."""
    indent = '        '
    f.write('    def _bytes(self) -> bytes:\n')
    f.write('        import io\n')
    f.write('        buf = io.BytesIO()\n')
    f.write(f'        buf.write(struct.pack(\'<I\', self.CONSTRUCTOR_ID))\n')

    # Handle flag computation
    has_flags = any(a.flag_indicator for a in obj.args)
    flag_fields = {}  # flag_name -> {bit_index: arg_name}
    for arg in obj.real_args:
        if arg.flag:
            flag_fields.setdefault(arg.flag, {})[arg.flag_index] = arg

    if has_flags:
        for flag_name, bits in flag_fields.items():
            f.write(f'        {flag_name} = 0\n')
            for bit, arg in bits.items():
                if arg.type == 'true':
                    f.write(f'        if self.{arg.name}:\n')
                    f.write(f'            {flag_name} |= (1 << {bit})\n')
                else:
                    f.write(f'        if self.{arg.name} is not None:\n')
                    f.write(f'            {flag_name} |= (1 << {bit})\n')

    for arg in obj.sorted_args() if hasattr(obj, 'sorted_args') else obj._sorted_args():
        if arg.generic_definition:
            continue
        if arg.flag_indicator:
            fname = arg.orig_name() if hasattr(arg, 'orig_name') else arg.name.rstrip('_').replace('is_self', 'self')
            f.write(f'        buf.write(struct.pack(\'<I\', {fname}))\n')
            continue

        lines = _args_serialization(arg, indent)
        for line in lines:
            f.write(line + '\n')

    f.write('        return buf.getvalue()\n\n')


def _write_from_reader(obj: TLObject, f):
    """Write the from_reader() classmethod."""
    f.write('    @classmethod\n')
    f.write('    def from_reader(cls, reader):\n')

    # Collect args
    real = [a for a in obj.args if not a.generic_definition]
    if not real:
        f.write('        return cls()\n\n')
        return

    has_flags = any(a.flag_indicator for a in real)
    flag_vars = set()

    f.write('        _args = {}\n')
    for arg in real:
        name = arg.name
        t = getattr(arg, 'type', None)

        if arg.flag_indicator:
            # Read flags int
            orig = arg.orig_name() if hasattr(arg, 'orig_name') else name
            f.write(f'        {orig} = reader.read_int(signed=False)\n')
            flag_vars.add(orig)
            continue

        if arg.flag:
            # Conditional read
            bit = arg.flag_index
            fname = arg.flag
            # 'true' flags only exist in the bitmask - no data to read
            if arg.type == 'true':
                f.write(f'        _args[\'{name}\'] = bool({fname} & (1 << {bit}))\n')
                continue
            f.write(f'        if {fname} & (1 << {bit}):\n')
            read_lines = _read_single_type(name, t, arg.is_vector,
                                           arg.use_vector_id,
                                           '            ')
            for line in read_lines:
                f.write(line + '\n')
            f.write(f'        else:\n')
            f.write(f'            _args[\'{name}\'] = None\n')
        else:
            read_lines = _read_single_type(name, t, arg.is_vector,
                                           arg.use_vector_id,
                                           '        ')
            for line in read_lines:
                f.write(line + '\n')

    f.write('        return cls(**_args)\n\n')


def _read_single_type(name, t, is_vector, use_vector_id, indent) -> List[str]:
    lines = []

    def w(code):
        lines.append(f'{indent}{code}')

    def assign(code):
        lines.append(f'{indent}_args[\'{name}\'] = {code}')

    if is_vector:
        if use_vector_id:
            w('reader.read_int(signed=False)  # skip vector id')
        w(f'_count_{name} = reader.read_int()')
        w(f'_list_{name} = []')
        w(f'for _ in range(_count_{name}):')
        item_lines = _read_single_item(f'_item_{name}', t, indent + '    ')
        lines.extend(item_lines)
        w(f'    _list_{name}.append(_item_{name})')
        assign(f'_list_{name}')
        return lines

    lines.extend(_read_single_item(f'_val_{name}', t, indent))
    assign(f'_val_{name}')
    return lines


def _read_single_item(var, t, indent) -> List[str]:
    lines = []

    def w(code):
        lines.append(f'{indent}{code}')

    if t in ('int', 'date'):
        w(f'{var} = reader.read_int()')
    elif t == 'long':
        w(f'{var} = reader.read_long()')
    elif t == 'int128':
        w(f'{var} = reader.read_large_int(128)')
    elif t == 'int256':
        w(f'{var} = reader.read_large_int(256)')
    elif t == 'double':
        w(f'{var} = reader.read_double()')
    elif t == 'string':
        w(f'{var} = reader.tgread_string()')
    elif t == 'bytes':
        w(f'{var} = reader.tgread_bytes()')
    elif t == 'Bool':
        w(f'{var} = reader.tgread_bool()')
    elif t == 'true':
        w(f'{var} = True')
    else:
        w(f'{var} = reader.tgread_object()')

    return lines


def _write_class(obj: TLObject, kind: str, f):
    """Write a complete class definition for a TLObject."""
    from zlib import crc32
    subclass_id = crc32(obj.result.encode('ascii')) & 0xFFFFFFFF

    f.write(f'\nclass {obj.class_name}({kind}):\n')
    f.write(f'    """TL type: {obj.fullname}"""\n')
    f.write(f'    CONSTRUCTOR_ID = {obj.id:#010x}\n')
    f.write(f'    SUBCLASS_OF_ID = {subclass_id:#010x}\n\n')

    # __init__
    if obj.real_args:
        # Build parameter list
        required = [a for a in obj.real_args if not a.flag and not a.can_be_inferred]
        optional = [a for a in obj.real_args if a.flag or a.can_be_inferred]

        params = []
        for a in required:
            params.append(f'{a.name}: {a.type_hint()}')
        for a in optional:
            params.append(f'{a.name}: {a.type_hint()} = None')

        params_str = ', '.join(params)
        f.write(f'    def __init__(self, {params_str}):\n')
        for a in obj.real_args:
            f.write(f'        self.{a.name} = {a.name}\n')
        f.write('\n')
    else:
        f.write('    def __init__(self):\n')
        f.write('        pass\n\n')

    # to_dict
    f.write('    def to_dict(self):\n')
    f.write(f'        return {{\n')
    f.write(f"            '_': '{obj.class_name}',\n")
    for a in obj.real_args:
        f.write(f"            '{a.name}': self.{a.name},\n")
    f.write('        }\n\n')

    # _bytes
    _write_to_bytes(obj, f)

    # from_reader
    _write_from_reader(obj, f)


def generate_tl_modules(
    tl_objects: List[TLObject],
    out_dir: str,
    depth: int = 2
):
    """
    Generate Python modules from TLObjects.

    Args:
        tl_objects: list from parse_tl()
        out_dir: root of tglib package (e.g. tglib/)
        depth: how many levels deep the generated files are from package root
    """
    types_dir = Path(out_dir) / 'tl' / 'types'
    funcs_dir = Path(out_dir) / 'tl' / 'functions'
    types_dir.mkdir(parents=True, exist_ok=True)
    funcs_dir.mkdir(parents=True, exist_ok=True)

    # Separate types and functions, group by namespace
    types_by_ns = defaultdict(list)
    funcs_by_ns = defaultdict(list)

    for obj in tl_objects:
        ns = obj.namespace or ''
        if obj.is_function:
            funcs_by_ns[ns].append(obj)
        else:
            types_by_ns[ns].append(obj)

    base = '.' * depth

    def write_module(ns_map, out_d, kind):
        all_classes = []
        ns_list = sorted(ns_map.keys())

        for ns in ns_list:
            objs = sorted(ns_map[ns], key=lambda x: x.name)
            filename = f'{ns}.py' if ns else '_root.py'
            filepath = out_d / filename

            with open(filepath, 'w', encoding='utf-8') as f:
                f.write(f'{AUTO_GEN_NOTICE}\n')
                f.write(f'from {base}tlobject import TLObject\n')
                if kind == 'TLRequest':
                    f.write(f'from {base}tlobject import TLRequest\n')
                f.write('from typing import Optional, List, Union\n')
                f.write('import os\nimport struct\nfrom datetime import datetime\n\n')

                for obj in objs:
                    _write_class(obj, kind, f)
                    all_classes.append((obj, ns))

            if ns:
                # Ensure namespace __init__.py exists
                ns_init = out_d / ns / '__init__.py'

        return ns_list, all_classes

    t_ns, t_classes = write_module(types_by_ns, types_dir, 'TLObject')
    f_ns, f_classes = write_module(funcs_by_ns, funcs_dir, 'TLRequest')

    # Generate types/__init__.py with namespace imports
    with open(types_dir / '__init__.py', 'w', encoding='utf-8') as f:
        f.write(f'{AUTO_GEN_NOTICE}\n')
        # Import root-namespace types from _root.py (avoids circular self-import)
        if '' in types_by_ns:
            objs = types_by_ns['']
            names = ', '.join(o.class_name for o in sorted(objs, key=lambda x: x.name))
            if names:
                f.write(f'from ._root import {names}  # noqa\n')
        for ns in sorted(n for n in types_by_ns if n):
            f.write(f'from . import {ns}  # noqa\n')

    with open(funcs_dir / '__init__.py', 'w', encoding='utf-8') as f:
        f.write(f'{AUTO_GEN_NOTICE}\n')
        # Import root-namespace functions from _root.py
        if '' in funcs_by_ns:
            objs = funcs_by_ns['']
            names = ', '.join(o.class_name for o in sorted(objs, key=lambda x: x.name))
            if names:
                f.write(f'from ._root import {names}  # noqa\n')
        for ns in sorted(n for n in funcs_by_ns if n):
            f.write(f'from . import {ns}  # noqa\n')

    # Generate alltlobjects.py
    all_objects_path = Path(out_dir) / 'tl' / 'alltlobjects.py'
    with open(all_objects_path, 'w', encoding='utf-8') as f:
        f.write(f'{AUTO_GEN_NOTICE}\n')
        f.write('# Maps constructor_id -> class for all TL types and functions\n\n')

        # imports for types
        for ns, objs in sorted(types_by_ns.items()):
            if ns:
                module = f'.types.{ns}'
            else:
                module = '.types._root'
            names = ', '.join(o.class_name for o in sorted(objs, key=lambda x: x.name))
            if names:
                f.write(f'from {module} import {names}\n')

        for ns, objs in sorted(funcs_by_ns.items()):
            if ns:
                module = f'.functions.{ns}'
            else:
                module = '.functions._root'
            names = ', '.join(o.class_name for o in sorted(objs, key=lambda x: x.name))
            if names:
                f.write(f'from {module} import {names}\n')

        f.write('\ntlobjects = {\n')
        for obj, ns in t_classes:
            f.write(f'    {obj.id:#010x}: {obj.class_name},\n')
        for obj, ns in f_classes:
            f.write(f'    {obj.id:#010x}: {obj.class_name},\n')
        f.write('}\n')

    print(f'✅ Generated {len(t_classes)} types and {len(f_classes)} functions')
    print(f'   Types dir:     {types_dir}')
    print(f'   Functions dir: {funcs_dir}')
    print(f'   All objects:   {all_objects_path}')



"""
MTProtoPlainSender - sends/receives unencrypted messages during auth key generation.
https://core.telegram.org/mtproto/description#unencrypted-messages
"""
import struct

from .mtprotostate import MTProtoState
from ..errors import InvalidBufferError
from ..extensions import BinaryReader


class MTProtoPlainSender:
    """
    Sends and receives plain (unencrypted) MTProto messages.
    Used only during the auth key generation handshake.
    """

    def __init__(self, connection, *, loggers=None):
        self._connection = connection
        self._state = MTProtoState(auth_key=None, loggers=loggers or {})

    async def send(self, request):
        """Serialize request, send it, wait for response and deserialize it."""
        body = bytes(request)
        msg_id = self._state._get_new_msg_id()

        # auth_key_id (0) + msg_id + length + body
        packet = struct.pack('<qqi', 0, msg_id, len(body)) + body
        await self._connection.send(packet)

        data = await self._connection.recv()
        if len(data) < 20:
            raise InvalidBufferError(data)

        with BinaryReader(data) as reader:
            auth_key_id = reader.read_long()
            assert auth_key_id == 0, f'Expected auth_key_id=0, got {auth_key_id}'
            _msg_id = reader.read_long()
            _length = reader.read_int()
            return reader.tgread_object()



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class BotInfo(TLObject):
    """TL type: bots.botInfo"""
    CONSTRUCTOR_ID = 0xe8a775b0
    SUBCLASS_OF_ID = 0xca7b2235

    def __init__(self, name: str, about: str, description: str):
        self.name = name
        self.about = about
        self.description = description

    def to_dict(self):
        return {
            '_': 'BotInfo',
            'name': self.name,
            'about': self.about,
            'description': self.description,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.name))
        buf.write(TLObject.serialize_bytes(self.about))
        buf.write(TLObject.serialize_bytes(self.description))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_name = reader.tgread_string()
        _args['name'] = _val_name
        _val_about = reader.tgread_string()
        _args['about'] = _val_about
        _val_description = reader.tgread_string()
        _args['description'] = _val_description
        return cls(**_args)


class PopularAppBots(TLObject):
    """TL type: bots.popularAppBots"""
    CONSTRUCTOR_ID = 0x1991b13b
    SUBCLASS_OF_ID = 0x7b64be7d

    def __init__(self, users: List['TypeUser'], next_offset: Optional[str] = None):
        self.users = users
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'PopularAppBots',
            'users': self.users,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class PreviewInfo(TLObject):
    """TL type: bots.previewInfo"""
    CONSTRUCTOR_ID = 0x0ca71d64
    SUBCLASS_OF_ID = 0xf0c27f35

    def __init__(self, media: List['TypeBotPreviewMedia'], lang_codes: List[str]):
        self.media = media
        self.lang_codes = lang_codes

    def to_dict(self):
        return {
            '_': 'PreviewInfo',
            'media': self.media,
            'lang_codes': self.lang_codes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.media)))
        for item in self.media:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.lang_codes)))
        for item in self.lang_codes:
            buf.write(TLObject.serialize_bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_media = reader.read_int()
        _list_media = []
        for _ in range(_count_media):
            _item_media = reader.tgread_object()
            _list_media.append(_item_media)
        _args['media'] = _list_media
        reader.read_int(signed=False)  # skip vector id
        _count_lang_codes = reader.read_int()
        _list_lang_codes = []
        for _ in range(_count_lang_codes):
            _item_lang_codes = reader.tgread_string()
            _list_lang_codes.append(_item_lang_codes)
        _args['lang_codes'] = _list_lang_codes
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class AppConfig(TLObject):
    """TL type: help.appConfig"""
    CONSTRUCTOR_ID = 0xdd18782e
    SUBCLASS_OF_ID = 0x14381c9a

    def __init__(self, hash: int, config: 'TypeJSONValue'):
        self.hash = hash
        self.config = config

    def to_dict(self):
        return {
            '_': 'AppConfig',
            'hash': self.hash,
            'config': self.config,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.hash))
        buf.write(bytes(self.config))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        _val_config = reader.tgread_object()
        _args['config'] = _val_config
        return cls(**_args)


class AppConfigNotModified(TLObject):
    """TL type: help.appConfigNotModified"""
    CONSTRUCTOR_ID = 0x7cde641d
    SUBCLASS_OF_ID = 0x14381c9a

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'AppConfigNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class AppUpdate(TLObject):
    """TL type: help.appUpdate"""
    CONSTRUCTOR_ID = 0xccbbce30
    SUBCLASS_OF_ID = 0x5897069e

    def __init__(self, id: int, version: str, text: str, entities: List['TypeMessageEntity'], can_not_skip: Optional[bool] = None, document: Optional['TypeDocument'] = None, url: Optional[str] = None, sticker: Optional['TypeDocument'] = None):
        self.id = id
        self.version = version
        self.text = text
        self.entities = entities
        self.can_not_skip = can_not_skip
        self.document = document
        self.url = url
        self.sticker = sticker

    def to_dict(self):
        return {
            '_': 'AppUpdate',
            'id': self.id,
            'version': self.version,
            'text': self.text,
            'entities': self.entities,
            'can_not_skip': self.can_not_skip,
            'document': self.document,
            'url': self.url,
            'sticker': self.sticker,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.can_not_skip:
            flags |= (1 << 0)
        if self.document is not None:
            flags |= (1 << 1)
        if self.url is not None:
            flags |= (1 << 2)
        if self.sticker is not None:
            flags |= (1 << 3)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.id))
        buf.write(TLObject.serialize_bytes(self.version))
        buf.write(TLObject.serialize_bytes(self.text))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.entities)))
        for item in self.entities:
            buf.write(bytes(self.item))
        if self.document is not None:
            buf.write(bytes(self.document))
        if self.url is not None:
            buf.write(TLObject.serialize_bytes(self.url))
        if self.sticker is not None:
            buf.write(bytes(self.sticker))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['can_not_skip'] = bool(flags & (1 << 0))
        _val_id = reader.read_int()
        _args['id'] = _val_id
        _val_version = reader.tgread_string()
        _args['version'] = _val_version
        _val_text = reader.tgread_string()
        _args['text'] = _val_text
        reader.read_int(signed=False)  # skip vector id
        _count_entities = reader.read_int()
        _list_entities = []
        for _ in range(_count_entities):
            _item_entities = reader.tgread_object()
            _list_entities.append(_item_entities)
        _args['entities'] = _list_entities
        if flags & (1 << 1):
            _val_document = reader.tgread_object()
            _args['document'] = _val_document
        else:
            _args['document'] = None
        if flags & (1 << 2):
            _val_url = reader.tgread_string()
            _args['url'] = _val_url
        else:
            _args['url'] = None
        if flags & (1 << 3):
            _val_sticker = reader.tgread_object()
            _args['sticker'] = _val_sticker
        else:
            _args['sticker'] = None
        return cls(**_args)


class CountriesList(TLObject):
    """TL type: help.countriesList"""
    CONSTRUCTOR_ID = 0x87d0759e
    SUBCLASS_OF_ID = 0xea31fe88

    def __init__(self, countries: List['TypeCountry'], hash: int):
        self.countries = countries
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'CountriesList',
            'countries': self.countries,
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.countries)))
        for item in self.countries:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_countries = reader.read_int()
        _list_countries = []
        for _ in range(_count_countries):
            _item_countries = reader.tgread_object()
            _list_countries.append(_item_countries)
        _args['countries'] = _list_countries
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        return cls(**_args)


class CountriesListNotModified(TLObject):
    """TL type: help.countriesListNotModified"""
    CONSTRUCTOR_ID = 0x93cc1f32
    SUBCLASS_OF_ID = 0xea31fe88

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'CountriesListNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class Country(TLObject):
    """TL type: help.country"""
    CONSTRUCTOR_ID = 0xc3878e23
    SUBCLASS_OF_ID = 0xa22e9e28

    def __init__(self, iso2: str, default_name: str, country_codes: List['TypeCountryCode'], hidden: Optional[bool] = None, name: Optional[str] = None):
        self.iso2 = iso2
        self.default_name = default_name
        self.country_codes = country_codes
        self.hidden = hidden
        self.name = name

    def to_dict(self):
        return {
            '_': 'Country',
            'iso2': self.iso2,
            'default_name': self.default_name,
            'country_codes': self.country_codes,
            'hidden': self.hidden,
            'name': self.name,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.hidden:
            flags |= (1 << 0)
        if self.name is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.iso2))
        buf.write(TLObject.serialize_bytes(self.default_name))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.country_codes)))
        for item in self.country_codes:
            buf.write(bytes(self.item))
        if self.name is not None:
            buf.write(TLObject.serialize_bytes(self.name))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['hidden'] = bool(flags & (1 << 0))
        _val_iso2 = reader.tgread_string()
        _args['iso2'] = _val_iso2
        _val_default_name = reader.tgread_string()
        _args['default_name'] = _val_default_name
        if flags & (1 << 1):
            _val_name = reader.tgread_string()
            _args['name'] = _val_name
        else:
            _args['name'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_country_codes = reader.read_int()
        _list_country_codes = []
        for _ in range(_count_country_codes):
            _item_country_codes = reader.tgread_object()
            _list_country_codes.append(_item_country_codes)
        _args['country_codes'] = _list_country_codes
        return cls(**_args)


class CountryCode(TLObject):
    """TL type: help.countryCode"""
    CONSTRUCTOR_ID = 0x4203c5ef
    SUBCLASS_OF_ID = 0x76f34665

    def __init__(self, country_code: str, prefixes: Optional[List[str]] = None, patterns: Optional[List[str]] = None):
        self.country_code = country_code
        self.prefixes = prefixes
        self.patterns = patterns

    def to_dict(self):
        return {
            '_': 'CountryCode',
            'country_code': self.country_code,
            'prefixes': self.prefixes,
            'patterns': self.patterns,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.prefixes is not None:
            flags |= (1 << 0)
        if self.patterns is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.country_code))
        if self.prefixes is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.prefixes)))
            for item in self.prefixes:
                buf.write(TLObject.serialize_bytes(self.item))
        if self.patterns is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.patterns)))
            for item in self.patterns:
                buf.write(TLObject.serialize_bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_country_code = reader.tgread_string()
        _args['country_code'] = _val_country_code
        if flags & (1 << 0):
            reader.read_int(signed=False)  # skip vector id
            _count_prefixes = reader.read_int()
            _list_prefixes = []
            for _ in range(_count_prefixes):
                _item_prefixes = reader.tgread_string()
                _list_prefixes.append(_item_prefixes)
            _args['prefixes'] = _list_prefixes
        else:
            _args['prefixes'] = None
        if flags & (1 << 1):
            reader.read_int(signed=False)  # skip vector id
            _count_patterns = reader.read_int()
            _list_patterns = []
            for _ in range(_count_patterns):
                _item_patterns = reader.tgread_string()
                _list_patterns.append(_item_patterns)
            _args['patterns'] = _list_patterns
        else:
            _args['patterns'] = None
        return cls(**_args)


class DeepLinkInfo(TLObject):
    """TL type: help.deepLinkInfo"""
    CONSTRUCTOR_ID = 0x6a4ee832
    SUBCLASS_OF_ID = 0x984aac38

    def __init__(self, message: str, update_app: Optional[bool] = None, entities: Optional[List['TypeMessageEntity']] = None):
        self.message = message
        self.update_app = update_app
        self.entities = entities

    def to_dict(self):
        return {
            '_': 'DeepLinkInfo',
            'message': self.message,
            'update_app': self.update_app,
            'entities': self.entities,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.update_app:
            flags |= (1 << 0)
        if self.entities is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.message))
        if self.entities is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.entities)))
            for item in self.entities:
                buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['update_app'] = bool(flags & (1 << 0))
        _val_message = reader.tgread_string()
        _args['message'] = _val_message
        if flags & (1 << 1):
            reader.read_int(signed=False)  # skip vector id
            _count_entities = reader.read_int()
            _list_entities = []
            for _ in range(_count_entities):
                _item_entities = reader.tgread_object()
                _list_entities.append(_item_entities)
            _args['entities'] = _list_entities
        else:
            _args['entities'] = None
        return cls(**_args)


class DeepLinkInfoEmpty(TLObject):
    """TL type: help.deepLinkInfoEmpty"""
    CONSTRUCTOR_ID = 0x66afa166
    SUBCLASS_OF_ID = 0x984aac38

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'DeepLinkInfoEmpty',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class InviteText(TLObject):
    """TL type: help.inviteText"""
    CONSTRUCTOR_ID = 0x18cb9f78
    SUBCLASS_OF_ID = 0xcf70aa35

    def __init__(self, message: str):
        self.message = message

    def to_dict(self):
        return {
            '_': 'InviteText',
            'message': self.message,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.message))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_message = reader.tgread_string()
        _args['message'] = _val_message
        return cls(**_args)


class NoAppUpdate(TLObject):
    """TL type: help.noAppUpdate"""
    CONSTRUCTOR_ID = 0xc45a6536
    SUBCLASS_OF_ID = 0x5897069e

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'NoAppUpdate',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class PassportConfig(TLObject):
    """TL type: help.passportConfig"""
    CONSTRUCTOR_ID = 0xa098d6af
    SUBCLASS_OF_ID = 0xc666c0ad

    def __init__(self, hash: int, countries_langs: 'TypeDataJSON'):
        self.hash = hash
        self.countries_langs = countries_langs

    def to_dict(self):
        return {
            '_': 'PassportConfig',
            'hash': self.hash,
            'countries_langs': self.countries_langs,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.hash))
        buf.write(bytes(self.countries_langs))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        _val_countries_langs = reader.tgread_object()
        _args['countries_langs'] = _val_countries_langs
        return cls(**_args)


class PassportConfigNotModified(TLObject):
    """TL type: help.passportConfigNotModified"""
    CONSTRUCTOR_ID = 0xbfb9f457
    SUBCLASS_OF_ID = 0xc666c0ad

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'PassportConfigNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class PeerColorOption(TLObject):
    """TL type: help.peerColorOption"""
    CONSTRUCTOR_ID = 0xadec6ebe
    SUBCLASS_OF_ID = 0x56b8ae98

    def __init__(self, color_id: int, hidden: Optional[bool] = None, colors: Optional['TypePeerColorSet'] = None, dark_colors: Optional['TypePeerColorSet'] = None, channel_min_level: Optional[int] = None, group_min_level: Optional[int] = None):
        self.color_id = color_id
        self.hidden = hidden
        self.colors = colors
        self.dark_colors = dark_colors
        self.channel_min_level = channel_min_level
        self.group_min_level = group_min_level

    def to_dict(self):
        return {
            '_': 'PeerColorOption',
            'color_id': self.color_id,
            'hidden': self.hidden,
            'colors': self.colors,
            'dark_colors': self.dark_colors,
            'channel_min_level': self.channel_min_level,
            'group_min_level': self.group_min_level,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.hidden:
            flags |= (1 << 0)
        if self.colors is not None:
            flags |= (1 << 1)
        if self.dark_colors is not None:
            flags |= (1 << 2)
        if self.channel_min_level is not None:
            flags |= (1 << 3)
        if self.group_min_level is not None:
            flags |= (1 << 4)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.color_id))
        if self.colors is not None:
            buf.write(bytes(self.colors))
        if self.dark_colors is not None:
            buf.write(bytes(self.dark_colors))
        if self.channel_min_level is not None:
            buf.write(struct.pack('<i', self.channel_min_level))
        if self.group_min_level is not None:
            buf.write(struct.pack('<i', self.group_min_level))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['hidden'] = bool(flags & (1 << 0))
        _val_color_id = reader.read_int()
        _args['color_id'] = _val_color_id
        if flags & (1 << 1):
            _val_colors = reader.tgread_object()
            _args['colors'] = _val_colors
        else:
            _args['colors'] = None
        if flags & (1 << 2):
            _val_dark_colors = reader.tgread_object()
            _args['dark_colors'] = _val_dark_colors
        else:
            _args['dark_colors'] = None
        if flags & (1 << 3):
            _val_channel_min_level = reader.read_int()
            _args['channel_min_level'] = _val_channel_min_level
        else:
            _args['channel_min_level'] = None
        if flags & (1 << 4):
            _val_group_min_level = reader.read_int()
            _args['group_min_level'] = _val_group_min_level
        else:
            _args['group_min_level'] = None
        return cls(**_args)


class PeerColorProfileSet(TLObject):
    """TL type: help.peerColorProfileSet"""
    CONSTRUCTOR_ID = 0x767d61eb
    SUBCLASS_OF_ID = 0x11cbe12c

    def __init__(self, palette_colors: List[int], bg_colors: List[int], story_colors: List[int]):
        self.palette_colors = palette_colors
        self.bg_colors = bg_colors
        self.story_colors = story_colors

    def to_dict(self):
        return {
            '_': 'PeerColorProfileSet',
            'palette_colors': self.palette_colors,
            'bg_colors': self.bg_colors,
            'story_colors': self.story_colors,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.palette_colors)))
        for item in self.palette_colors:
            buf.write(struct.pack('<i', self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.bg_colors)))
        for item in self.bg_colors:
            buf.write(struct.pack('<i', self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.story_colors)))
        for item in self.story_colors:
            buf.write(struct.pack('<i', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_palette_colors = reader.read_int()
        _list_palette_colors = []
        for _ in range(_count_palette_colors):
            _item_palette_colors = reader.read_int()
            _list_palette_colors.append(_item_palette_colors)
        _args['palette_colors'] = _list_palette_colors
        reader.read_int(signed=False)  # skip vector id
        _count_bg_colors = reader.read_int()
        _list_bg_colors = []
        for _ in range(_count_bg_colors):
            _item_bg_colors = reader.read_int()
            _list_bg_colors.append(_item_bg_colors)
        _args['bg_colors'] = _list_bg_colors
        reader.read_int(signed=False)  # skip vector id
        _count_story_colors = reader.read_int()
        _list_story_colors = []
        for _ in range(_count_story_colors):
            _item_story_colors = reader.read_int()
            _list_story_colors.append(_item_story_colors)
        _args['story_colors'] = _list_story_colors
        return cls(**_args)


class PeerColorSet(TLObject):
    """TL type: help.peerColorSet"""
    CONSTRUCTOR_ID = 0x26219a58
    SUBCLASS_OF_ID = 0x11cbe12c

    def __init__(self, colors: List[int]):
        self.colors = colors

    def to_dict(self):
        return {
            '_': 'PeerColorSet',
            'colors': self.colors,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.colors)))
        for item in self.colors:
            buf.write(struct.pack('<i', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_colors = reader.read_int()
        _list_colors = []
        for _ in range(_count_colors):
            _item_colors = reader.read_int()
            _list_colors.append(_item_colors)
        _args['colors'] = _list_colors
        return cls(**_args)


class PeerColors(TLObject):
    """TL type: help.peerColors"""
    CONSTRUCTOR_ID = 0x00f8ed08
    SUBCLASS_OF_ID = 0x0e3f6733

    def __init__(self, hash: int, colors: List['TypePeerColorOption']):
        self.hash = hash
        self.colors = colors

    def to_dict(self):
        return {
            '_': 'PeerColors',
            'hash': self.hash,
            'colors': self.colors,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.hash))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.colors)))
        for item in self.colors:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        reader.read_int(signed=False)  # skip vector id
        _count_colors = reader.read_int()
        _list_colors = []
        for _ in range(_count_colors):
            _item_colors = reader.tgread_object()
            _list_colors.append(_item_colors)
        _args['colors'] = _list_colors
        return cls(**_args)


class PeerColorsNotModified(TLObject):
    """TL type: help.peerColorsNotModified"""
    CONSTRUCTOR_ID = 0x2ba1f5ce
    SUBCLASS_OF_ID = 0x0e3f6733

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'PeerColorsNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class PremiumPromo(TLObject):
    """TL type: help.premiumPromo"""
    CONSTRUCTOR_ID = 0x5334759c
    SUBCLASS_OF_ID = 0xc987a338

    def __init__(self, status_text: str, status_entities: List['TypeMessageEntity'], video_sections: List[str], videos: List['TypeDocument'], period_options: List['TypePremiumSubscriptionOption'], users: List['TypeUser']):
        self.status_text = status_text
        self.status_entities = status_entities
        self.video_sections = video_sections
        self.videos = videos
        self.period_options = period_options
        self.users = users

    def to_dict(self):
        return {
            '_': 'PremiumPromo',
            'status_text': self.status_text,
            'status_entities': self.status_entities,
            'video_sections': self.video_sections,
            'videos': self.videos,
            'period_options': self.period_options,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.status_text))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.status_entities)))
        for item in self.status_entities:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.video_sections)))
        for item in self.video_sections:
            buf.write(TLObject.serialize_bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.videos)))
        for item in self.videos:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.period_options)))
        for item in self.period_options:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_status_text = reader.tgread_string()
        _args['status_text'] = _val_status_text
        reader.read_int(signed=False)  # skip vector id
        _count_status_entities = reader.read_int()
        _list_status_entities = []
        for _ in range(_count_status_entities):
            _item_status_entities = reader.tgread_object()
            _list_status_entities.append(_item_status_entities)
        _args['status_entities'] = _list_status_entities
        reader.read_int(signed=False)  # skip vector id
        _count_video_sections = reader.read_int()
        _list_video_sections = []
        for _ in range(_count_video_sections):
            _item_video_sections = reader.tgread_string()
            _list_video_sections.append(_item_video_sections)
        _args['video_sections'] = _list_video_sections
        reader.read_int(signed=False)  # skip vector id
        _count_videos = reader.read_int()
        _list_videos = []
        for _ in range(_count_videos):
            _item_videos = reader.tgread_object()
            _list_videos.append(_item_videos)
        _args['videos'] = _list_videos
        reader.read_int(signed=False)  # skip vector id
        _count_period_options = reader.read_int()
        _list_period_options = []
        for _ in range(_count_period_options):
            _item_period_options = reader.tgread_object()
            _list_period_options.append(_item_period_options)
        _args['period_options'] = _list_period_options
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class PromoData(TLObject):
    """TL type: help.promoData"""
    CONSTRUCTOR_ID = 0x08a4d87a
    SUBCLASS_OF_ID = 0x9d595542

    def __init__(self, expires: Optional[datetime], pending_suggestions: List[str], dismissed_suggestions: List[str], chats: List['TypeChat'], users: List['TypeUser'], proxy: Optional[bool] = None, peer: Optional['TypePeer'] = None, psa_type: Optional[str] = None, psa_message: Optional[str] = None, custom_pending_suggestion: Optional['TypePendingSuggestion'] = None):
        self.expires = expires
        self.pending_suggestions = pending_suggestions
        self.dismissed_suggestions = dismissed_suggestions
        self.chats = chats
        self.users = users
        self.proxy = proxy
        self.peer = peer
        self.psa_type = psa_type
        self.psa_message = psa_message
        self.custom_pending_suggestion = custom_pending_suggestion

    def to_dict(self):
        return {
            '_': 'PromoData',
            'expires': self.expires,
            'pending_suggestions': self.pending_suggestions,
            'dismissed_suggestions': self.dismissed_suggestions,
            'chats': self.chats,
            'users': self.users,
            'proxy': self.proxy,
            'peer': self.peer,
            'psa_type': self.psa_type,
            'psa_message': self.psa_message,
            'custom_pending_suggestion': self.custom_pending_suggestion,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.proxy:
            flags |= (1 << 0)
        if self.peer is not None:
            flags |= (1 << 3)
        if self.psa_type is not None:
            flags |= (1 << 1)
        if self.psa_message is not None:
            flags |= (1 << 2)
        if self.custom_pending_suggestion is not None:
            flags |= (1 << 4)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.expires))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.pending_suggestions)))
        for item in self.pending_suggestions:
            buf.write(TLObject.serialize_bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.dismissed_suggestions)))
        for item in self.dismissed_suggestions:
            buf.write(TLObject.serialize_bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.peer is not None:
            buf.write(bytes(self.peer))
        if self.psa_type is not None:
            buf.write(TLObject.serialize_bytes(self.psa_type))
        if self.psa_message is not None:
            buf.write(TLObject.serialize_bytes(self.psa_message))
        if self.custom_pending_suggestion is not None:
            buf.write(bytes(self.custom_pending_suggestion))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['proxy'] = bool(flags & (1 << 0))
        _val_expires = reader.read_int()
        _args['expires'] = _val_expires
        if flags & (1 << 3):
            _val_peer = reader.tgread_object()
            _args['peer'] = _val_peer
        else:
            _args['peer'] = None
        if flags & (1 << 1):
            _val_psa_type = reader.tgread_string()
            _args['psa_type'] = _val_psa_type
        else:
            _args['psa_type'] = None
        if flags & (1 << 2):
            _val_psa_message = reader.tgread_string()
            _args['psa_message'] = _val_psa_message
        else:
            _args['psa_message'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_pending_suggestions = reader.read_int()
        _list_pending_suggestions = []
        for _ in range(_count_pending_suggestions):
            _item_pending_suggestions = reader.tgread_string()
            _list_pending_suggestions.append(_item_pending_suggestions)
        _args['pending_suggestions'] = _list_pending_suggestions
        reader.read_int(signed=False)  # skip vector id
        _count_dismissed_suggestions = reader.read_int()
        _list_dismissed_suggestions = []
        for _ in range(_count_dismissed_suggestions):
            _item_dismissed_suggestions = reader.tgread_string()
            _list_dismissed_suggestions.append(_item_dismissed_suggestions)
        _args['dismissed_suggestions'] = _list_dismissed_suggestions
        if flags & (1 << 4):
            _val_custom_pending_suggestion = reader.tgread_object()
            _args['custom_pending_suggestion'] = _val_custom_pending_suggestion
        else:
            _args['custom_pending_suggestion'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class PromoDataEmpty(TLObject):
    """TL type: help.promoDataEmpty"""
    CONSTRUCTOR_ID = 0x98f6ac75
    SUBCLASS_OF_ID = 0x9d595542

    def __init__(self, expires: Optional[datetime]):
        self.expires = expires

    def to_dict(self):
        return {
            '_': 'PromoDataEmpty',
            'expires': self.expires,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.expires))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_expires = reader.read_int()
        _args['expires'] = _val_expires
        return cls(**_args)


class RecentMeUrls(TLObject):
    """TL type: help.recentMeUrls"""
    CONSTRUCTOR_ID = 0x0e0310d7
    SUBCLASS_OF_ID = 0xf269c477

    def __init__(self, urls: List['TypeRecentMeUrl'], chats: List['TypeChat'], users: List['TypeUser']):
        self.urls = urls
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'RecentMeUrls',
            'urls': self.urls,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.urls)))
        for item in self.urls:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_urls = reader.read_int()
        _list_urls = []
        for _ in range(_count_urls):
            _item_urls = reader.tgread_object()
            _list_urls.append(_item_urls)
        _args['urls'] = _list_urls
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class Support(TLObject):
    """TL type: help.support"""
    CONSTRUCTOR_ID = 0x17c6b5f6
    SUBCLASS_OF_ID = 0x7159bceb

    def __init__(self, phone_number: str, user: 'TypeUser'):
        self.phone_number = phone_number
        self.user = user

    def to_dict(self):
        return {
            '_': 'Support',
            'phone_number': self.phone_number,
            'user': self.user,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.phone_number))
        buf.write(bytes(self.user))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_phone_number = reader.tgread_string()
        _args['phone_number'] = _val_phone_number
        _val_user = reader.tgread_object()
        _args['user'] = _val_user
        return cls(**_args)


class SupportName(TLObject):
    """TL type: help.supportName"""
    CONSTRUCTOR_ID = 0x8c05f1c9
    SUBCLASS_OF_ID = 0x7f50b7c2

    def __init__(self, name: str):
        self.name = name

    def to_dict(self):
        return {
            '_': 'SupportName',
            'name': self.name,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.name))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_name = reader.tgread_string()
        _args['name'] = _val_name
        return cls(**_args)


class TermsOfService(TLObject):
    """TL type: help.termsOfService"""
    CONSTRUCTOR_ID = 0x780a0310
    SUBCLASS_OF_ID = 0x20ee8312

    def __init__(self, id: 'TypeDataJSON', text: str, entities: List['TypeMessageEntity'], popup: Optional[bool] = None, min_age_confirm: Optional[int] = None):
        self.id = id
        self.text = text
        self.entities = entities
        self.popup = popup
        self.min_age_confirm = min_age_confirm

    def to_dict(self):
        return {
            '_': 'TermsOfService',
            'id': self.id,
            'text': self.text,
            'entities': self.entities,
            'popup': self.popup,
            'min_age_confirm': self.min_age_confirm,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.popup:
            flags |= (1 << 0)
        if self.min_age_confirm is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.id))
        buf.write(TLObject.serialize_bytes(self.text))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.entities)))
        for item in self.entities:
            buf.write(bytes(self.item))
        if self.min_age_confirm is not None:
            buf.write(struct.pack('<i', self.min_age_confirm))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['popup'] = bool(flags & (1 << 0))
        _val_id = reader.tgread_object()
        _args['id'] = _val_id
        _val_text = reader.tgread_string()
        _args['text'] = _val_text
        reader.read_int(signed=False)  # skip vector id
        _count_entities = reader.read_int()
        _list_entities = []
        for _ in range(_count_entities):
            _item_entities = reader.tgread_object()
            _list_entities.append(_item_entities)
        _args['entities'] = _list_entities
        if flags & (1 << 1):
            _val_min_age_confirm = reader.read_int()
            _args['min_age_confirm'] = _val_min_age_confirm
        else:
            _args['min_age_confirm'] = None
        return cls(**_args)


class TermsOfServiceUpdate(TLObject):
    """TL type: help.termsOfServiceUpdate"""
    CONSTRUCTOR_ID = 0x28ecf961
    SUBCLASS_OF_ID = 0x293c2977

    def __init__(self, expires: Optional[datetime], terms_of_service: 'TypeTermsOfService'):
        self.expires = expires
        self.terms_of_service = terms_of_service

    def to_dict(self):
        return {
            '_': 'TermsOfServiceUpdate',
            'expires': self.expires,
            'terms_of_service': self.terms_of_service,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.expires))
        buf.write(bytes(self.terms_of_service))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_expires = reader.read_int()
        _args['expires'] = _val_expires
        _val_terms_of_service = reader.tgread_object()
        _args['terms_of_service'] = _val_terms_of_service
        return cls(**_args)


class TermsOfServiceUpdateEmpty(TLObject):
    """TL type: help.termsOfServiceUpdateEmpty"""
    CONSTRUCTOR_ID = 0xe3309f7f
    SUBCLASS_OF_ID = 0x293c2977

    def __init__(self, expires: Optional[datetime]):
        self.expires = expires

    def to_dict(self):
        return {
            '_': 'TermsOfServiceUpdateEmpty',
            'expires': self.expires,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.expires))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_expires = reader.read_int()
        _args['expires'] = _val_expires
        return cls(**_args)


class TimezonesList(TLObject):
    """TL type: help.timezonesList"""
    CONSTRUCTOR_ID = 0x7b74ed71
    SUBCLASS_OF_ID = 0xca76e475

    def __init__(self, timezones: List['TypeTimezone'], hash: int):
        self.timezones = timezones
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'TimezonesList',
            'timezones': self.timezones,
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.timezones)))
        for item in self.timezones:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_timezones = reader.read_int()
        _list_timezones = []
        for _ in range(_count_timezones):
            _item_timezones = reader.tgread_object()
            _list_timezones.append(_item_timezones)
        _args['timezones'] = _list_timezones
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        return cls(**_args)


class TimezonesListNotModified(TLObject):
    """TL type: help.timezonesListNotModified"""
    CONSTRUCTOR_ID = 0x970708cc
    SUBCLASS_OF_ID = 0xca76e475

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'TimezonesListNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class UserInfo(TLObject):
    """TL type: help.userInfo"""
    CONSTRUCTOR_ID = 0x01eb3758
    SUBCLASS_OF_ID = 0x5c53d7d8

    def __init__(self, message: str, entities: List['TypeMessageEntity'], author: str, date: Optional[datetime]):
        self.message = message
        self.entities = entities
        self.author = author
        self.date = date

    def to_dict(self):
        return {
            '_': 'UserInfo',
            'message': self.message,
            'entities': self.entities,
            'author': self.author,
            'date': self.date,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.message))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.entities)))
        for item in self.entities:
            buf.write(bytes(self.item))
        buf.write(TLObject.serialize_bytes(self.author))
        buf.write(struct.pack('<i', self.date))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_message = reader.tgread_string()
        _args['message'] = _val_message
        reader.read_int(signed=False)  # skip vector id
        _count_entities = reader.read_int()
        _list_entities = []
        for _ in range(_count_entities):
            _item_entities = reader.tgread_object()
            _list_entities.append(_item_entities)
        _args['entities'] = _list_entities
        _val_author = reader.tgread_string()
        _args['author'] = _val_author
        _val_date = reader.read_int()
        _args['date'] = _val_date
        return cls(**_args)


class UserInfoEmpty(TLObject):
    """TL type: help.userInfoEmpty"""
    CONSTRUCTOR_ID = 0xf3ae2eed
    SUBCLASS_OF_ID = 0x5c53d7d8

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'UserInfoEmpty',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class FileGif(TLObject):
    """TL type: storage.fileGif"""
    CONSTRUCTOR_ID = 0xcae1aadf
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FileGif',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class FileJpeg(TLObject):
    """TL type: storage.fileJpeg"""
    CONSTRUCTOR_ID = 0x007efe0e
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FileJpeg',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class FileMov(TLObject):
    """TL type: storage.fileMov"""
    CONSTRUCTOR_ID = 0x4b09ebbc
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FileMov',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class FileMp3(TLObject):
    """TL type: storage.fileMp3"""
    CONSTRUCTOR_ID = 0x528a0677
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FileMp3',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class FileMp4(TLObject):
    """TL type: storage.fileMp4"""
    CONSTRUCTOR_ID = 0xb3cea0e4
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FileMp4',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class FilePartial(TLObject):
    """TL type: storage.filePartial"""
    CONSTRUCTOR_ID = 0x40bc6f52
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FilePartial',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class FilePdf(TLObject):
    """TL type: storage.filePdf"""
    CONSTRUCTOR_ID = 0xae1e508d
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FilePdf',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class FilePng(TLObject):
    """TL type: storage.filePng"""
    CONSTRUCTOR_ID = 0x0a4f63c0
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FilePng',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class FileUnknown(TLObject):
    """TL type: storage.fileUnknown"""
    CONSTRUCTOR_ID = 0xaa963b05
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FileUnknown',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class FileWebp(TLObject):
    """TL type: storage.fileWebp"""
    CONSTRUCTOR_ID = 0x1081464c
    SUBCLASS_OF_ID = 0xf3a1e6f3

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'FileWebp',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()




"""TLMessage - represents a single MTProto message."""


class TLMessage:
    """Represents an MTProto message with msg_id, seq_no and obj."""
    SIZE_OVERHEAD = 12  # msg_id (8) + seq_no (4) + length (4) ... wait 12?

    def __init__(self, msg_id: int, seq_no: int, obj):
        self.msg_id = msg_id
        self.seq_no = seq_no
        self.obj = obj

    def __repr__(self):
        return f'TLMessage(msg_id={self.msg_id}, seq_no={self.seq_no}, obj={self.obj})'



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class DeletePhotosRequest(TLRequest):
    """TL type: photos.deletePhotos"""
    CONSTRUCTOR_ID = 0x87cf7f2f
    SUBCLASS_OF_ID = 0x8918e168

    def __init__(self, id: List['TypeInputPhoto']):
        self.id = id

    def to_dict(self):
        return {
            '_': 'DeletePhotosRequest',
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.id)))
        for item in self.id:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_id = reader.read_int()
        _list_id = []
        for _ in range(_count_id):
            _item_id = reader.tgread_object()
            _list_id.append(_item_id)
        _args['id'] = _list_id
        return cls(**_args)


class GetUserPhotosRequest(TLRequest):
    """TL type: photos.getUserPhotos"""
    CONSTRUCTOR_ID = 0x91cd32a8
    SUBCLASS_OF_ID = 0x27cfb967

    def __init__(self, user_id: 'TypeInputUser', offset: int, max_id: int, limit: int):
        self.user_id = user_id
        self.offset = offset
        self.max_id = max_id
        self.limit = limit

    def to_dict(self):
        return {
            '_': 'GetUserPhotosRequest',
            'user_id': self.user_id,
            'offset': self.offset,
            'max_id': self.max_id,
            'limit': self.limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.user_id))
        buf.write(struct.pack('<i', self.offset))
        buf.write(struct.pack('<q', self.max_id))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        _val_offset = reader.read_int()
        _args['offset'] = _val_offset
        _val_max_id = reader.read_long()
        _args['max_id'] = _val_max_id
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class UpdateProfilePhotoRequest(TLRequest):
    """TL type: photos.updateProfilePhoto"""
    CONSTRUCTOR_ID = 0x09e82039
    SUBCLASS_OF_ID = 0xc292bd24

    def __init__(self, id: 'TypeInputPhoto', fallback: Optional[bool] = None, bot: Optional['TypeInputUser'] = None):
        self.id = id
        self.fallback = fallback
        self.bot = bot

    def to_dict(self):
        return {
            '_': 'UpdateProfilePhotoRequest',
            'id': self.id,
            'fallback': self.fallback,
            'bot': self.bot,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.fallback:
            flags |= (1 << 0)
        if self.bot is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.id))
        if self.bot is not None:
            buf.write(bytes(self.bot))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['fallback'] = bool(flags & (1 << 0))
        if flags & (1 << 1):
            _val_bot = reader.tgread_object()
            _args['bot'] = _val_bot
        else:
            _args['bot'] = None
        _val_id = reader.tgread_object()
        _args['id'] = _val_id
        return cls(**_args)


class UploadContactProfilePhotoRequest(TLRequest):
    """TL type: photos.uploadContactProfilePhoto"""
    CONSTRUCTOR_ID = 0xe14c4a71
    SUBCLASS_OF_ID = 0xc292bd24

    def __init__(self, user_id: 'TypeInputUser', suggest: Optional[bool] = None, save: Optional[bool] = None, file: Optional['TypeInputFile'] = None, video: Optional['TypeInputFile'] = None, video_start_ts: Optional[float] = None, video_emoji_markup: Optional['TypeVideoSize'] = None):
        self.user_id = user_id
        self.suggest = suggest
        self.save = save
        self.file = file
        self.video = video
        self.video_start_ts = video_start_ts
        self.video_emoji_markup = video_emoji_markup

    def to_dict(self):
        return {
            '_': 'UploadContactProfilePhotoRequest',
            'user_id': self.user_id,
            'suggest': self.suggest,
            'save': self.save,
            'file': self.file,
            'video': self.video,
            'video_start_ts': self.video_start_ts,
            'video_emoji_markup': self.video_emoji_markup,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.suggest:
            flags |= (1 << 3)
        if self.save:
            flags |= (1 << 4)
        if self.file is not None:
            flags |= (1 << 0)
        if self.video is not None:
            flags |= (1 << 1)
        if self.video_start_ts is not None:
            flags |= (1 << 2)
        if self.video_emoji_markup is not None:
            flags |= (1 << 5)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.user_id))
        if self.file is not None:
            buf.write(bytes(self.file))
        if self.video is not None:
            buf.write(bytes(self.video))
        if self.video_start_ts is not None:
            buf.write(struct.pack('<d', self.video_start_ts))
        if self.video_emoji_markup is not None:
            buf.write(bytes(self.video_emoji_markup))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['suggest'] = bool(flags & (1 << 3))
        _args['save'] = bool(flags & (1 << 4))
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        if flags & (1 << 0):
            _val_file = reader.tgread_object()
            _args['file'] = _val_file
        else:
            _args['file'] = None
        if flags & (1 << 1):
            _val_video = reader.tgread_object()
            _args['video'] = _val_video
        else:
            _args['video'] = None
        if flags & (1 << 2):
            _val_video_start_ts = reader.read_double()
            _args['video_start_ts'] = _val_video_start_ts
        else:
            _args['video_start_ts'] = None
        if flags & (1 << 5):
            _val_video_emoji_markup = reader.tgread_object()
            _args['video_emoji_markup'] = _val_video_emoji_markup
        else:
            _args['video_emoji_markup'] = None
        return cls(**_args)


class UploadProfilePhotoRequest(TLRequest):
    """TL type: photos.uploadProfilePhoto"""
    CONSTRUCTOR_ID = 0x0388a3b5
    SUBCLASS_OF_ID = 0xc292bd24

    def __init__(self, fallback: Optional[bool] = None, bot: Optional['TypeInputUser'] = None, file: Optional['TypeInputFile'] = None, video: Optional['TypeInputFile'] = None, video_start_ts: Optional[float] = None, video_emoji_markup: Optional['TypeVideoSize'] = None):
        self.fallback = fallback
        self.bot = bot
        self.file = file
        self.video = video
        self.video_start_ts = video_start_ts
        self.video_emoji_markup = video_emoji_markup

    def to_dict(self):
        return {
            '_': 'UploadProfilePhotoRequest',
            'fallback': self.fallback,
            'bot': self.bot,
            'file': self.file,
            'video': self.video,
            'video_start_ts': self.video_start_ts,
            'video_emoji_markup': self.video_emoji_markup,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.fallback:
            flags |= (1 << 3)
        if self.bot is not None:
            flags |= (1 << 5)
        if self.file is not None:
            flags |= (1 << 0)
        if self.video is not None:
            flags |= (1 << 1)
        if self.video_start_ts is not None:
            flags |= (1 << 2)
        if self.video_emoji_markup is not None:
            flags |= (1 << 4)
        buf.write(struct.pack('<I', flags))
        if self.bot is not None:
            buf.write(bytes(self.bot))
        if self.file is not None:
            buf.write(bytes(self.file))
        if self.video is not None:
            buf.write(bytes(self.video))
        if self.video_start_ts is not None:
            buf.write(struct.pack('<d', self.video_start_ts))
        if self.video_emoji_markup is not None:
            buf.write(bytes(self.video_emoji_markup))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['fallback'] = bool(flags & (1 << 3))
        if flags & (1 << 5):
            _val_bot = reader.tgread_object()
            _args['bot'] = _val_bot
        else:
            _args['bot'] = None
        if flags & (1 << 0):
            _val_file = reader.tgread_object()
            _args['file'] = _val_file
        else:
            _args['file'] = None
        if flags & (1 << 1):
            _val_video = reader.tgread_object()
            _args['video'] = _val_video
        else:
            _args['video'] = None
        if flags & (1 << 2):
            _val_video_start_ts = reader.read_double()
            _args['video_start_ts'] = _val_video_start_ts
        else:
            _args['video_start_ts'] = None
        if flags & (1 << 4):
            _val_video_emoji_markup = reader.tgread_object()
            _args['video_emoji_markup'] = _val_video_emoji_markup
        else:
            _args['video_emoji_markup'] = None
        return cls(**_args)




"""
tglib error hierarchy.
"""


class TglibError(Exception):
    """Base exception for all tglib errors."""


class TypeNotFoundError(TglibError):
    """Raised when a TL constructor ID is unknown."""
    def __init__(self, constructor_id: int, remaining: bytes = b''):
        self.constructor_id = constructor_id
        self.invalid_constructor_id = constructor_id  # alias used by mtprotosender
        self.remaining = remaining
        super().__init__(
            f'Unknown TL constructor {constructor_id:#010x}. '
            f'Telegram is using a newer layer; update the TL schema.'
        )


class SecurityError(TglibError):
    """Raised when a security check fails (nonce mismatch, invalid hash, etc.)."""


class InvalidBufferError(TglibError):
    """Raised when a received buffer is invalid or too short."""
    def __init__(self, payload: bytes):
        self.payload = payload
        super().__init__(f'Invalid or too-short buffer ({len(payload)} bytes)')


class AuthKeyNotFound(TglibError):
    """Raised when the server doesn't recognize our auth key."""


class BadMessageError(TglibError):
    """Raised when Telegram sends a bad_msg_notification."""
    CODES = {
        16: 'msg_id too low (invalid time offset?)',
        17: 'msg_id too high (invalid time offset?)',
        18: 'incorrect two lower order msg_id bits',
        19: 'container msg_id is the same as msg_id of a previously received message',
        20: 'message too old, and it cannot be verified whether the server has received a message with this msg_id or not',
        32: 'msg_seqno too low',
        33: 'msg_seqno too high',
        34: 'an even msg_seqno expected (for non-content-related message)',
        35: 'odd msg_seqno expected (for content-related message)',
        48: 'incorrect server salt',
        64: 'invalid container',
    }

    def __init__(self, code: int):
        self.code = code
        description = self.CODES.get(code, f'Unknown error code {code}')
        super().__init__(f'Bad message notification {code}: {description}')


class RPCError(TglibError):
    """RPC error returned by Telegram servers."""
    def __init__(self, code: int, message: str, request=None):
        self.code = code
        self.message = message
        self.request = request
        super().__init__(f'RPCError {code}: {message}')


class FloodWaitError(RPCError):
    """FLOOD_WAIT_X - must wait X seconds."""
    def __init__(self, seconds: int, request=None):
        self.seconds = seconds
        super().__init__(420, f'FLOOD_WAIT_{seconds}', request)


class AuthKeyUnregisteredError(RPCError):
    def __init__(self, request=None):
        super().__init__(401, 'AUTH_KEY_UNREGISTERED', request)


class UserNotParticipantError(RPCError):
    def __init__(self, request=None):
        super().__init__(400, 'USER_NOT_PARTICIPANT', request)


class PhoneNumberInvalidError(RPCError):
    def __init__(self, request=None):
        super().__init__(400, 'PHONE_NUMBER_INVALID', request)


class PasswordHashInvalidError(RPCError):
    """Wrong 2FA password — the SRP proof was rejected by Telegram."""
    def __init__(self, request=None):
        super().__init__(400, 'PASSWORD_HASH_INVALID', request)


class SessionPasswordNeededError(RPCError):
    """2FA password required."""
    def __init__(self, request=None):
        super().__init__(401, 'SESSION_PASSWORD_NEEDED', request)


def rpc_message_to_error(code: int, message: str, request=None) -> RPCError:
    """Convert an RPC error code+message into the most specific exception."""
    import re

    if code == 420:
        m = re.match(r'FLOOD_WAIT_(\d+)', message)
        if m:
            return FloodWaitError(int(m.group(1)), request)

    mapping = {
        'AUTH_KEY_UNREGISTERED': AuthKeyUnregisteredError,
        'SESSION_PASSWORD_NEEDED': SessionPasswordNeededError,
        'PHONE_NUMBER_INVALID': PhoneNumberInvalidError,
        'USER_NOT_PARTICIPANT': UserNotParticipantError,
        'PASSWORD_HASH_INVALID': PasswordHashInvalidError,
    }
    cls = mapping.get(message)
    if cls:
        return cls(request)

    return RPCError(code, message, request)
